package com.yunzhi.niucai.common;

import com.yunzhi.niucai.config.SMSConfig;
import lombok.Data;
import lombok.experimental.Accessors;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

@Slf4j
@Component
public class SMSUtils {
    @Autowired
    SMSConfig smsConfig;

    public void sendMessage(Message message) throws Exception {
        request(message);
    }

    private void request(Message message) throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity entity = new HttpEntity(message, headers);

        try {
            RestTemplate request = new RestTemplate();
            ResponseEntity<String> response = request.exchange(
                    smsConfig.getApi(),
                    HttpMethod.POST,
                    entity,
                    String.class,
                    new Object()
            );

            if (null == response || null == response.getStatusCode()) {
                throw new Exception("发送短信失败, 未知异常");
            }

            if (HttpStatus.OK.value() != response.getStatusCodeValue()) {
                throw new Exception("发送短信失败, 网络异常");
            }

            String body = response.getBody();
            if (!StringUtils.isEmpty(body)) {
                if (!(body.indexOf("成功") > -1 || body.indexOf("success") > -1)) {
                    throw new Exception("发送短信失败, " + body);
                }
            }

        } catch (RestClientException e) {
            log.error("发送短信消息失败 {}", e.getMessage());
            throw e;
        }

        return;
    }

    @Data
    @Accessors(chain = true)
    public static class Message {
        String code;
        String tel;
        String[] params;
    }
}
