package com.yunzhi.niucai.controller;

import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipayOpenAuthTokenAppRequest;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.response.AlipayOpenAuthTokenAppResponse;
import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.CommConstants;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.entity.TaOrder;
import com.yunzhi.niucai.entity.TaShopApp;
import com.yunzhi.niucai.service.ITaOrderService;
import com.yunzhi.niucai.service.ITaShopAppService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigInteger;
import java.security.SecureRandom;
import java.time.LocalDateTime;


@Api(tags = "登入/登出")
@RestController
@RequestMapping("/aliPay")
public class AliPayController extends BaseController {

    @Autowired
    public ITaShopAppService iTaShopAppService;

    @Autowired
    private ITaOrderService taOrderService;

    @Value("${aliPay.notifyUrl}")
    private String aliPayNotifyUrl;

    /**
     * 手机app获取唤醒支付宝的参数
     * @param request
     * @param taOrder
     * @return
     * @throws Exception
     */
    @PostMapping("/create")
    @ApiOperation(value = "创建订单", notes = "创建订单", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean create(HttpServletRequest request, @RequestBody TaOrder taOrder) throws Exception {
        //检查token的使用时间
        TaShopApp taShopApp = iTaShopAppService.getOne(new QueryWrapper<TaShopApp>().eq("shop_id", taOrder.getShopId()));
        checkAliPayToken(taShopApp);
        String outTradeNo = "ALI" + System.currentTimeMillis() + randomString(5);
        AlipayClient alipayClient = new DefaultAlipayClient("https://openapi.alipay.com/gateway.do", taShopApp.getAppid(), taShopApp.getMerchantPrivateKey(), "json", "GBK", taShopApp.getAlipayPublicKey(), "RSA2");
        AlipayTradeAppPayRequest aliPayRequest = new AlipayTradeAppPayRequest();
        aliPayRequest.putOtherTextParam("app_auth_token", taShopApp.getAppAuthToken());
        aliPayRequest.setNotifyUrl(aliPayNotifyUrl);
        aliPayRequest.setBizContent("{" +
                "\"timeout_express\":\"30m\"," +
                "\"total_amount\":\""+taOrder.getAmount() / 100 +"\"," +
                "\"subject\":\"支付宝支付\"," +
                "\"out_trade_no\":\""+outTradeNo+"\"," +
                "  }");
        AlipayTradeAppPayResponse response = alipayClient.sdkExecute(aliPayRequest);
        if(response.isSuccess()){
            // 更新商户订单号并改为支付中
            taOrder.setOutTradeNo(outTradeNo);
            taOrder.setStatus(CommConstants.SRTATUS_PAYING);
            taOrder.setUpdateDate(LocalDateTime.now());
            taOrderService.updateById(taOrder);
            // 更新token是最后使用时间
            taShopApp.setUpdateTokenDate(LocalDateTime.now());
            iTaShopAppService.updateById(taShopApp);
        } else {
            System.out.println("调用失败");
        }
        return ResponseBean.success(response);
    }

    /**
     * 检查alipay的token是否24小时未使用
     * @param taShopApp
     * @return
     */
    private void checkAliPayToken(TaShopApp taShopApp) throws AlipayApiException {
        if (taShopApp.getUpdateTokenDate().plusDays(1).isBefore(LocalDateTime.now())){
            return;
        }
        getAliPayToken(taShopApp,"refresh_token");
    }

    /**
     * 获取或刷新支付宝token
     * @param taShopApp
     * @param grantType
     * @param appAuthCode
     * @throws AlipayApiException
     */
    private void getAliPayToken(TaShopApp taShopApp,String grantType,String ...appAuthCode) throws AlipayApiException {
        AlipayClient alipayClient = new DefaultAlipayClient("https://openapi.alipay.com/gateway.do", taShopApp.getAppid(), taShopApp.getMerchantPrivateKey(), "json", "GBK", taShopApp.getAlipayPublicKey(), "RSA2");
        AlipayOpenAuthTokenAppRequest request = new AlipayOpenAuthTokenAppRequest();
        if (grantType.equals("refresh_token")){
            request.setBizContent("{" +
                    "\"grant_type\":\"refresh_token\"," +
                    "\"refresh_token\":\""+taShopApp.getAppRefreshToken()+"\"," +
                    "  }");
        }else if (grantType.equals("authorization_code")){
            request.setBizContent("{" +
                    "\"grant_type\":\"authorization_code\"," +
                    "\"code\":\""+appAuthCode+"\"," +
                    "  }");
        }

        AlipayOpenAuthTokenAppResponse response = alipayClient.execute(request);
        if(response.isSuccess()){
            System.out.println("调用成功");
            taShopApp.setAppAuthToken(response.getAppAuthToken());
            taShopApp.setAppRefreshToken(response.getAppRefreshToken());
            taShopApp.setUpdateTokenDate(LocalDateTime.now());
            iTaShopAppService.updateById(taShopApp);
        } else {
            System.out.println("调用失败");
        }
    }

    public String randomString(int length) {
        String str = new BigInteger(130, new SecureRandom()).toString(32);
        return str.substring(0, length);
    }

    /**
     * 支付成功回调
     * @param request
     */
    @GetMapping("/callback")
    public void callback(HttpServletRequest request){
        String tradeNo = request.getParameter("trade_no");
        String appId = request.getParameter("app_id");
        String outTradeNo = request.getParameter("out_trade_no");
        String tradeStatus = request.getParameter("trade_status");
        TaOrder taOrder = taOrderService.getOne(new QueryWrapper<TaOrder>().eq("out_trade_no", outTradeNo));
        taOrder.setUpdateDate(LocalDateTime.now());
        // 支付失败
        if (!tradeStatus.equals(CommConstants.TRADE_STATUS_SUCCESS)){
            taOrder.setStatus(CommConstants.SRTATUS_EFFECTIVE);
            taOrderService.updateById(taOrder);
            return;
        }
        //支付成功 更新为已支付状态
        taOrder.setIsPayed(true);
        taOrder.setStatus(CommConstants.SRTATUS_EFFECTIVE);
        taOrder.setTradeNo(tradeNo);
        taOrderService.updateById(taOrder);
    }

    /**
     * 应用授权成功回调
     * @param appId
     * @param appAuthCode
     * @throws AlipayApiException
     */
    @GetMapping("/auth/notice")
    public void aliPayAuthNotice(@RequestParam("app_id") String appId,@RequestParam("app_auth_code") String appAuthCode) throws AlipayApiException {
        TaShopApp taShopApp =iTaShopAppService.getOne(new QueryWrapper<TaShopApp>().eq("appid",appId));
        getAliPayToken(taShopApp,"authorization_code",appAuthCode);
    }
}
