package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.common.*;
import com.yunzhi.niucai.entity.TaCustomer;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.service.ITaCustomerService;
import com.yunzhi.niucai.vo.LoginParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Api(tags = "登入/登出")
@RestController
public class LoginController extends BaseController {

    @Autowired
    ITaCustomerService iTaCustomerService;

    @PostMapping("/app/login")
    @ApiOperation(value="登录", notes = "登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean appLogin(@ApiParam("登录参数") @RequestBody LoginParam loginParam,
                              HttpServletResponse response) throws Exception {
        if (null == loginParam) {
            return ResponseBean.error("账户或密码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        String userName = loginParam.getUserName();
        String password = loginParam.getPassword();

        if (StringUtils.isEmpty(userName) || StringUtils.isEmpty(password)) {
            return ResponseBean.error("账户或密码不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        QueryWrapper<TaCustomer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", userName);
        queryWrapper.gt("status", StatusEnum.DELETED.getCode());
        TaCustomer taCustomer = iTaCustomerService.getOne(queryWrapper);
        if (null == taCustomer) {
            return ResponseBean.error("账户或密码不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (!checkPassword(password, taCustomer.getLoginPass(), taCustomer.getCustomerId().toString())) {
            return ResponseBean.error("账户或密码不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (!StatusEnum.NORMAL.getCode().equals(taCustomer.getStatus())) {
            return ResponseBean.error("用户状态不正确, 请联系管理人员", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 生成 token
        Map<String, Object> claims = new HashMap<String, Object>(){{
            put("id", taCustomer.getCustomerId());
            put("client", "app");
            put("exp", getTokenExpire());
        }};
        String jws = JWTUtils.encode(claims);
        response.addHeader(JWTUtils.AUTH_HEADER, jws);

        taCustomer.setLoginPass(null);
        taCustomer.setCashPass(null);

        return ResponseBean.success(taCustomer);
    }

    // 默认 token 7 天过期
    private Date getTokenExpire() {
        LocalDateTime now = LocalDateTime.now();
        return Date.from(now.plusDays(7).atZone(ZoneId.systemDefault()).toInstant());
    }

    private boolean checkPassword(String src, String targ, String salt) {
        return EncryptUtils.md5(src, salt).equals(targ);
    }
}
