package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.enums.StatusEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.niucai.service.ISysAdService;
import com.yunzhi.niucai.entity.SysAd;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 系统广告 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-08-27
 */

@Api(tags = "广告、通知、Banner")
@RestController
@RequestMapping("/")
public class SysAdController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysAdController.class);

    @Autowired
    public ISysAdService iSysAdService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/ad",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysAdList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                  @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                  @ApiParam(value = "广告类型: 可用类型查询 AdTypeEnum") @RequestParam(value ="adType", required = false) String adType,
                                  @ApiParam(value = "发布状态: 1 已发布, 0 未发布", allowableValues = "1,0") @RequestParam(value ="status", required = false) Integer status,
                                  @ApiParam(value = "客户端", allowableValues = "app, shop") @PathVariable(value ="client") String client) throws Exception{

		    IPage<SysAd> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<SysAd> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(null != adType, "ad_type", adType);
            queryWrapper.eq(null != status, "status", status);
            queryWrapper.gt("status", StatusEnum.DELETED.getCode());
            queryWrapper.orderByDesc("weight");
            queryWrapper.orderByDesc("create_date");

            IPage<SysAd> result = iSysAdService.page(pg, queryWrapper);
            return ResponseBean.success(result);
    }

//    /**
//     * 保存对象
//     * @param sysAd 实体对象
//     * @return
//     */
//    @RequestMapping(value="/sysAd",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean sysAdAdd(@ApiParam("保存内容") @RequestBody SysAd sysAd) throws Exception{
//
//        if (iSysAdService.save(sysAd)){
//            return ResponseBean.success(sysAd);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/sysAd/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean sysAdDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iSysAdService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param sysAd 实体对象
//     * @return
//     */
//    @RequestMapping(value="/sysAd/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean sysAdUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody SysAd sysAd) throws Exception{
//
//        if (iSysAdService.updateById(sysAd)){
//            return ResponseBean.success(iSysAdService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id查询对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/sysAd/{id}",method= RequestMethod.GET)
//    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean sysAdGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        return ResponseBean.success(iSysAdService.getById(id));
//    }
}
