package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.entity.TaCustomer;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.niucai.service.ITaCustomerBettingService;
import com.yunzhi.niucai.entity.TaCustomerBetting;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * <p>
    * 客户投注  前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-09-09
 */

@Api(tags = "客户投注 ")
@RestController
@RequestMapping("/")
public class TaCustomerBettingController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaCustomerBettingController.class);

    @Autowired
    public ITaCustomerBettingService iTaCustomerBettingService;


    /**
     * 查询投注列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/app/customer/betting/list",method= RequestMethod.GET)
    @ApiOperation(value="查询投注列表", notes = "App 端只允许查询自己的", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taCustomerBettingList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                              @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                              HttpServletRequest request) throws Exception{
            TaCustomer taCustomer = getCustomer(request);
            if (null == taCustomer) {
                return ResponseBean.error("账户不存在, 或重新登录", ResponseBean.ERROR_ILLEGAL_PARAMS);
            }

		    IPage<TaCustomerBetting> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaCustomerBetting> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("customer_id", taCustomer.getCustomerId());
            queryWrapper.orderByDesc("create_date");

            IPage<TaCustomerBetting> result = iTaCustomerBettingService.page(pg, queryWrapper);
            return ResponseBean.success(result);
    }
//
//    /**
//     * 保存对象
//     * @param taCustomerBetting 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taCustomerBetting",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean taCustomerBettingAdd(@ApiParam("保存内容") @RequestBody TaCustomerBetting taCustomerBetting) throws Exception{
//
//        if (iTaCustomerBettingService.save(taCustomerBetting)){
//            return ResponseBean.success(taCustomerBetting);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taCustomerBetting/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taCustomerBettingDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaCustomerBettingService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taCustomerBetting 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taCustomerBetting/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taCustomerBettingUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaCustomerBetting taCustomerBetting) throws Exception{
//
//        if (iTaCustomerBettingService.updateById(taCustomerBetting)){
//            return ResponseBean.success(iTaCustomerBettingService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
    /**
     * 查询具体投注内容
     * @param id  实体ID
     */
    @RequestMapping(value="/app/customer/betting/{id}",method= RequestMethod.GET)
    @ApiOperation(value="依据ID查询投注详情", notes = "依据ID查询投注详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taCustomerBettingGet(@ApiParam("投注ID") @PathVariable Integer id) throws Exception{
        Map<String, Object> result = iTaCustomerBettingService.getSummaryOf(id);
        return ResponseBean.success(result);
    }
}
