package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.entity.TaCustomer;
import com.yunzhi.niucai.enums.StatusEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.niucai.service.ITaShopService;
import com.yunzhi.niucai.entity.TaShop;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * <p>
    * 店铺 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-08-28
 */

@Api(tags = "店铺")
@RestController
@RequestMapping("/")
public class TaShopController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaShopController.class);

    @Autowired
    public ITaShopService iTaShopService;


    @GetMapping("/app/shop/available")
    @ApiOperation(value = "查询用用店铺", notes = "查询用用店铺", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getAvailableList(@ApiParam("彩种类型: 可用值请查阅 彩种字典") @RequestParam String lotteryId,
                                         @ApiParam("当前地点: 经度(lng),纬度(lat) 半角逗号连接") @RequestParam String location,
                                         HttpServletRequest request) throws Exception{
        if (location.indexOf(",") < 0) {
            return ResponseBean.error("地点参数格式不正确", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaCustomer customer = getCustomer(request);
        Map<String, Object> result = iTaShopService.getAvailableList(customer, lotteryId, location);
        return ResponseBean.success(result);
    }

//    /**
//     * 分页查询列表
//     * @param pageNum
//     * @param pageSize
//     * @return
//     */
//    @RequestMapping(value="/app/shop",method= RequestMethod.GET)
//    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taShopList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
//                                   @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{
//
//		    IPage<TaShop> pg = new Page<>(pageNum, pageSize);
//            QueryWrapper<TaShop> queryWrapper = new QueryWrapper<>();
//            queryWrapper.orderByDesc("create_date");
//
//            IPage<TaShop> result = iTaShopService.page(pg, queryWrapper);
//            return ResponseBean.success(result);
//    }
//
//    /**
//     * 保存对象
//     * @param taShop 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taShop",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean taShopAdd(@ApiParam("保存内容") @RequestBody TaShop taShop) throws Exception{
//
//        if (iTaShopService.save(taShop)){
//            return ResponseBean.success(taShop);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taShop/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taShopDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaShopService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taShop 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taShop/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taShopUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaShop taShop) throws Exception{
//
//        if (iTaShopService.updateById(taShop)){
//            return ResponseBean.success(iTaShopService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id查询对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taShop/{id}",method= RequestMethod.GET)
//    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taShopGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        return ResponseBean.success(iTaShopService.getById(id));
//    }
}
