package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.yunzhi.niucai.common.EncryptUtils;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.TaCustomer;
import com.yunzhi.niucai.entity.TaShop;
import com.yunzhi.niucai.enums.ReferralCodeEnum;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaCustomerMapper;
import com.yunzhi.niucai.service.ITaCustomerAccountService;
import com.yunzhi.niucai.service.ITaCustomerService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yunzhi.niucai.service.ITaShopService;
import com.yunzhi.niucai.vo.SignUpParam;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 客户 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-08-28
 */
@Service
public class TaCustomerServiceImpl extends ServiceImpl<TaCustomerMapper, TaCustomer> implements ITaCustomerService {

    @Autowired
    ITaShopService iTaShopService;

    @Autowired
    ITaCustomerAccountService iTaCustomerAccountService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean signUp(SignUpParam signUpParam) throws Exception {
        TaCustomer taCustomer = new TaCustomer();
        taCustomer.setName(signUpParam.getName());
        taCustomer.setPhone(signUpParam.getPhone());
        taCustomer.setReferralCode(signUpParam.getReferralCode());
        taCustomer.setStatus(StatusEnum.NORMAL.getCode());

        // 手机号唯一
        TaCustomer customerByPhone = getCustomerByPhone(signUpParam.getPhone());
        if (null != customerByPhone) {
            throw new Exception("当前手机号已经注册");
        }

        // 生成自己的邀请码
        boolean exists = true;
        while (exists) {
            String inviteCode = ReferralCodeEnum.CUSTOMER_START.getCode() + "-" + EncryptUtils.randDigitStr(6);
            TaCustomer exitCust = getByInviteCode(inviteCode);
            if (exitCust == null) {
                exists = false;
                taCustomer.setInviteCode(inviteCode);
            }
        }

        if (!save(taCustomer)) {
            throw new Exception("生成个人信息失败");
        }

        // 获取最新信息, 因为有部分字段是自动生成的
        taCustomer = getById(taCustomer.getCustomerId());

        // 更新推荐人或者店铺
        if (!StringUtils.isEmpty(signUpParam.getReferralCode())) {
            // C 开头说明是客户
            if (signUpParam.getReferralCode().startsWith(ReferralCodeEnum.CUSTOMER_START.getCode())) {
                try {
                    TaCustomer recCust = getByInviteCode(signUpParam.getReferralCode());

                    if (null != recCust) {
                        taCustomer.setRecPerson(recCust.getCustomerId());
                    }
                } catch (Exception e) {}
            }

            // S 开头说明是店铺
            if (signUpParam.getReferralCode().startsWith(ReferralCodeEnum.SHOP_START.getCode())) {
                try {
                    TaShop taShop = iTaShopService.getByInviteCode(signUpParam.getReferralCode());

                    if (null != taShop) {
                        taCustomer.setRecShop(taShop.getShopId());
                    }
                } catch (Exception e) {}
            }
        }

        // 更新密码
        taCustomer.setLoginPass(EncryptUtils.md5(signUpParam.getPassword(), taCustomer.getCustomerId().toString()));

        if (!updateById(taCustomer)) {
            throw new Exception("生成个人信息失败");
        }

        // 生成账户信息
        if (!iTaCustomerAccountService.newAccountBy(taCustomer)) {
            throw new Exception("生成个人账户信息失败");
        }

        return true;
    }

    @Override
    public TaCustomer getCustomerByPhone(String phone) throws Exception {
        QueryWrapper<TaCustomer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", phone);
        queryWrapper.gt("status", StatusEnum.DELETED.getCode());
        return getOne(queryWrapper);
    }

    @Override
    public boolean updateIdCard(TaCustomer taCustomer) throws Exception {
        UpdateWrapper<TaCustomer> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("real_name", taCustomer.getRealName());
        updateWrapper.set("id_card", taCustomer.getIdCard());
        updateWrapper.eq("customer_id", taCustomer.getCustomerId());

        return update(updateWrapper);
    }

    private TaCustomer getByInviteCode(String inviteCode) throws Exception {
        QueryWrapper<TaCustomer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("invite_code", inviteCode);
        queryWrapper.gt("status", StatusEnum.DELETED.getCode());

        return getOne(queryWrapper);
    }
}
