package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.common.BizUtils;
import com.yunzhi.niucai.common.CommConstants;
import com.yunzhi.niucai.common.DateUtils;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.*;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaLotteryResultDetailMapper;
import com.yunzhi.niucai.mapper.TaLotteryResultMapper;
import com.yunzhi.niucai.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 数字彩开奖结果  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-08-27
 */
@Slf4j
@Service
public class TaLotteryResultServiceImpl extends ServiceImpl<TaLotteryResultMapper, TaLotteryResult> implements ITaLotteryResultService {

    @Autowired
    ITdLotteryService iTdLotteryService;

    @Autowired
    ITdLotteryOnSaleService iTdLotteryOnSaleService;

    @Autowired
    ITaCustomerBettingService iTaCustomerBettingService;

    @Autowired
    ITaCustomerBettingItemService iTaCustomerBettingItemService;

    @Autowired
    ITdLotteryPrizeService iTdLotteryPrizeService;

    @Autowired
    TaLotteryResultMapper taLotteryResultMapper;

    @Autowired
    TaLotteryResultDetailMapper taLotteryResultDetailMapper;

    @Override
    public List<TaLotteryResult> getLastResult() throws Exception {
        return taLotteryResultMapper.getLastResult();
    }

    @Override
    public TaLotteryResult getLastResultByLottery(TdLottery tdLottery) throws Exception {
        QueryWrapper<TaLotteryResult> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("lottery_id", tdLottery.getLotteryId());
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());
        queryWrapper.orderByDesc("opening_date");
        queryWrapper.last("limit 1");

        return getOne(queryWrapper);
    }

    @Override
    public TaLotteryResult getResultBy(TdLottery lottery, String today) throws Exception {
        return null;
    }

    /**
     * 校验客户是否中奖, 如果中奖，则进行中奖相关更新
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public void checkAndSetLotteryPrize() throws Exception {
        LocalDateTime now = LocalDateTime.now();
        String today = now.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        int weekDay = now.getDayOfWeek().getValue();
        List<TdLotteryOnSale> onSaleList = iTdLotteryOnSaleService.getByWeekDay(weekDay);
        if (null == onSaleList || onSaleList.size() == 0) {
            log.info(today + "未找到有开售设置");
            return;
        }

        for (TdLotteryOnSale onSale : onSaleList) {
            String lotteryId = onSale.getLotteryId();
            log.info(String.format("彩种 %s %s 开奖, 开始检测中奖人员", lotteryId, today));

            // 彩种字典
            TdLottery lottery = iTdLotteryService.getById(lotteryId);

            // 中奖设置
            List<TdLotteryPrize> prizeSettings = iTdLotteryPrizeService.getByLottery(lottery);
            if (null == prizeSettings || prizeSettings.size() == 0) {
                log.error(String.format("彩种 %s 没有开奖相关设置", lotteryId));
                continue;
            }

            // 当前彩种的开奖结果
            TaLotteryResult lotteryResult = getLotteryResultBy(onSale.getLotteryId(), today);
            if (null == lotteryResult) {
                log.error(String.format("彩种 %s 未抓取到开奖结果", lotteryId));
                continue;
            }

            if (null == lotteryResult.getDetailList() || lotteryResult.getDetailList().size() == 0) {
                log.error(String.format("彩种 %s 未抓取到开奖明细结果", lotteryId));
                continue;
            }

            // 当前彩种的投注列表
            List<TaCustomerBetting> customerBettingList = iTaCustomerBettingService.getAllUnOpenBettingsBy(lottery, lotteryResult);
            if (null == customerBettingList || customerBettingList.size() == 0) {
                log.info(String.format("彩种 %s 本期无人投注", lotteryId));
                continue;
            }

            // 遍历校验中奖情况
            for (TaCustomerBetting customerBetting : customerBettingList) {
                try {
                    if (lottery.getIsDigit()) {
                        validLotteryCustomer(lottery, prizeSettings, lotteryResult, customerBetting);
                    }
//                    switch (lotteryId) {
//                        case CommConstants.LOTTERY_LOTTERY:
//                            validLotteryCustomer(lottery, prizeSettings, lotteryResult, customerBetting);
//                            break;
//                        case CommConstants.LOTTERY_P3:
//                            validP3Customer(lottery, prizeSettings, lotteryResult, customerBetting);
//                            break;
//                        case CommConstants.LOTTERY_P5:
//                            validP5Customer(lottery, prizeSettings, lotteryResult, customerBetting);
//                            break;
//                        case CommConstants.LOTTERY_DOUBLE:
//                            validDoubleCustomer(lottery, prizeSettings, lotteryResult, customerBetting);
//                            break;
//                        case CommConstants.LOTTERY_FOOTBALL:
//                            validFootballCustomer(lottery, prizeSettings, lotteryResult, customerBetting);
//                            break;
//                        case CommConstants.LOTTERY_BASKETBALL:
//                            validBasketballCustomer(lottery, prizeSettings, lotteryResult, customerBetting);
//                            break;
//                        default:
//                            break;
//                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    // continue;
                }
            }
        }
    }

    private void validLotteryCustomer(TdLottery lottery, List<TdLotteryPrize> prizeSettings, TaLotteryResult lotteryResult, TaCustomerBetting customerBetting) throws Exception {
        TaBettingPlan taBettingPlan = customerBetting.getBettingPlan();
        if (null == taBettingPlan) {
            log.error(String.format("ID %s 的投注没有投注方案", customerBetting.getBettingId()));
            return;
        }

        if (null == taBettingPlan.getItemList() || taBettingPlan.getItemList().size() == 0
                || null == customerBetting.getItemList() || customerBetting.getItemList().size() == 0) {
            log.error(String.format("ID %s 的投注没有投注明细", customerBetting.getBettingId()));
            return;
        }

        int winTotalCharges = 0;
        for (TaBettingPlanItem item : taBettingPlan.getItemList()) {
            // 是否中奖
            boolean isWinning = false;
            // 中奖金额
            int winAmount = 0;

            try {
                TaCustomerBettingItem bettingItem = getBettingItemMap(customerBetting.getItemList(), item);
                if (bettingItem == null) {
                    log.error(String.format("ID %s 的投注查询映射投注明细失败", customerBetting.getBettingId()));
                    continue;
                }

                // 排列三直选, 排列五 都是依靠比对数字位是否一致来决定是否中奖的
                if (lottery.getLotteryId().equals(CommConstants.LOTTERY_P5)) {
                    isWinning = BizUtils.checkPrizeNumStrict(lotteryResult.getFirstResult().split(","), item.getFirstNums().split(","));
                    if (isWinning) {
                        TaLotteryResultDetail resultDetail = lotteryResult.getDetailList().get(0);
                        if (null == resultDetail) {
                            log.error(String.format("第 %s 期排列五结果明细未抓取", lotteryResult.getIssueNo()));
                            continue;
                        }

                        winAmount = Math.round(Float.parseFloat(resultDetail.getMoney()) * 100);
                    }
                } else if (lottery.getLotteryId().equals(CommConstants.LOTTERY_P3)) {
                    String[] bingoNums = lotteryResult.getFirstResult().split(",");
                    if (bingoNums.length != 3) {
                        log.error(String.format("第 %s 期排列三抓取结果不正确", lotteryResult.getIssueNo()));
                        continue;
                    }

                    String level = "直选";
                    if ("组选3".equals(item.getPlayWay())) {
                        level = "组三";
                    } else if ("组选6".equals(item.getPlayWay())) {
                        level = "组六";
                    }

                    TaLotteryResultDetail resultDetail = filterResultDetail(lotteryResult.getDetailList(), level);
                    if (null == resultDetail) {
                        log.error(String.format("第 %s 期排列三结果明细未抓取", lotteryResult.getIssueNo()));
                        continue;
                    }

                    // 直选是通过比对数字来校验中奖
                    if ("直选".equals(level)) {
                        isWinning = BizUtils.checkPrizeNumStrict(bingoNums, item.getFirstNums().split(","));
                        if (isWinning) {
                            winAmount = Math.round(Float.parseFloat(resultDetail.getMoney()) * 100);
                        }
                    } else {
                        // 出现2次的号码
                        boolean hasTwiceNum = bingoNums[0].equals(bingoNums[1])
                                || bingoNums[0].equals(bingoNums[2])
                                || bingoNums[1].equals(bingoNums[2]);

                        // 如果存在出现2次的号码, 只有可能中组选三, 否则只有可能中组选6
                        if (!hasTwiceNum) {
                            if ("组六".equals(level)) {
                                isWinning = item.getFirstNums().contains(bingoNums[0])
                                        && item.getFirstNums().contains(bingoNums[1])
                                        && item.getFirstNums().contains(bingoNums[2]);
                                if (isWinning) {
                                    winAmount = Math.round(Float.parseFloat(resultDetail.getMoney()) * 100);
                                }
                            }
                        } else {
                            if ("组三".equals(level)) {
                                // 如果是复式
                                if (item.getIsMulti()) {
                                    isWinning = item.getFirstNums().contains(bingoNums[0])
                                            && item.getFirstNums().contains(bingoNums[1])
                                            && item.getFirstNums().contains(bingoNums[2]);
                                    if (isWinning) {
                                        winAmount = Math.round(Float.parseFloat(resultDetail.getMoney()) * 100);
                                    }
                                } else {
                                    // 投注号
                                    String[] buyNums = item.getFirstNums().split(",");
                                    // 由小到大排序
                                    Arrays.sort(bingoNums);
                                    Arrays.sort(buyNums);

                                    isWinning = BizUtils.checkPrizeNumStrict(bingoNums, buyNums);
                                    if (isWinning) {
                                        winAmount = Math.round(Float.parseFloat(resultDetail.getMoney()) * 100);
                                    }
                                }
                            }
                        }
                    }

                } else {
                    // 大乐透  双色球
                    String firstNums = item.getFirstNums();
                    if (StringUtils.isEmpty(firstNums)) {
                        log.error(String.format("ID %s 的投注明细无前区内容", customerBetting.getBettingId()));
                        continue;
                    }

                    String firstDanNums = item.getFirstDan();
                    if (StringUtils.isEmpty(firstDanNums)) {
                        firstDanNums = "";
                    }

                    // 前区中奖数
                    int firstSameNums = BizUtils.getSameNums(
                            lotteryResult.getFirstResult().split(","),
                            firstNums.split(","),
                            firstDanNums.split(",")
                    );

                    String secondNums = item.getFirstNums();
                    if (StringUtils.isEmpty(secondNums)) {
                        log.error(String.format("ID %s 的投注明细无后区内容", customerBetting.getBettingId()));
                        continue;
                    }

                    String secondDanNums = item.getFirstDan();
                    if (StringUtils.isEmpty(secondDanNums)) {
                        secondDanNums = "";
                    }

                    // 后区中奖数
                    int secondSameNums = BizUtils.getSameNums(
                            lotteryResult.getSecondResult().split(","),
                            secondNums.split(","),
                            secondDanNums.split(","));

                    // 选择结果
                    String curNums = String.format("%d-%d", firstSameNums, secondSameNums);

                    boolean needContinue = false;
                    TdLotteryPrize bingo = null;
                    for(TdLotteryPrize prize : prizeSettings) {
                        if (StringUtils.isEmpty(prize.getBingoRule())) {
                            log.error(String.format("彩种 %s 的中奖结果规则未维护", lottery.getLotteryId()));
                            needContinue = true;
                            break;
                        }

                        if (prize.getBingoRule().contains(curNums)) {
                            bingo = prize;
                            break;
                        }
                    }

                    if (needContinue) {
                        continue;
                    }

                    if (null != bingo) {
                        log.info(String.format("恭喜 %s 购买 %s 中了 %s", customerBetting.getCustomerId(), lottery.getLotteryId(), bingo.getLevel()));
                        isWinning = true;
                        bettingItem.setWinLevel(bingo.getLevel());
                        TaLotteryResultDetail resultDetail = filterResultDetail(lotteryResult.getDetailList(), bingo.getLevel());
                        if (null == resultDetail) {
                            log.error(String.format("彩种 %s 的中奖结果明细未成功抓取", lottery.getLotteryId()));
                            continue;
                        }

                        winAmount = Math.round(Float.parseFloat(resultDetail.getMoney()) * 100);
                        // 是否追加
                        boolean isAdd = customerBetting.getIsAdd();
                        if (isAdd && !StringUtils.isEmpty(resultDetail.getXMoney())) {
                            winAmount += Math.round(Float.parseFloat(resultDetail.getXMoney()) * 100);
                        }
                    }
                }

                winTotalCharges += winAmount;
                bettingItem.setIsOpen(true);
                bettingItem.setOpeningDate(DateUtils.today());
                bettingItem.setIsWinning(isWinning);
                bettingItem.setWinAmount(winAmount);
                iTaCustomerBettingItemService.updateById(bettingItem);
            } catch (Exception e) {
                e.printStackTrace();
                continue;
            }
        }

        if (winTotalCharges > 0) {
            customerBetting.setIsWinning(true);
            customerBetting.setWinAmount(winTotalCharges * customerBetting.getBettingTimes());
        }
        customerBetting.setIsOpen(true);
        customerBetting.setOpeningDate(DateUtils.today());
        iTaCustomerBettingService.updateById(customerBetting);
    }

    private TaLotteryResultDetail filterResultDetail(List<TaLotteryResultDetail> detailList, String level) {
        for (TaLotteryResultDetail d: detailList) {
            if (d.getLevel().equals(level)) {
                return d;
            }
        }

        return null;
    }

    private TaCustomerBettingItem getBettingItemMap(List<TaCustomerBettingItem> bettingItemList, TaBettingPlanItem item) {
        for (TaCustomerBettingItem it : bettingItemList) {
            if (item.getItemNo().equals(it.getPlanItemNo())) {
                return it;
            }
        }
        return null;
    }

    public TaLotteryResult getLotteryResultBy(String lotteryId, String openingDate) throws Exception {
        QueryWrapper<TaLotteryResult> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("lottery_id", lotteryId);
        queryWrapper.eq("opening_date", openingDate);
        queryWrapper.eq("status", StatusEnum.NORMAL.getCode());

        TaLotteryResult result = getOne(queryWrapper);
        if (null != result) {
            QueryWrapper<TaLotteryResultDetail> query2 = new QueryWrapper<>();
            query2.eq("result_no", result.getSerialNo());
            query2.eq("status", StatusEnum.NORMAL.getCode());
            query2.orderByAsc("serial_no");
            result.setDetailList(taLotteryResultDetailMapper.selectList(query2));
        }

        return result;
    }
}
