package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yunzhi.niucai.common.BizUtils;
import com.yunzhi.niucai.common.DateUtils;
import com.yunzhi.niucai.entity.*;
import com.yunzhi.niucai.enums.SportOpeningStatusEnum;
import com.yunzhi.niucai.enums.SportStatusEnum;
import com.yunzhi.niucai.mapper.TaCustomerBettingMapper;
import com.yunzhi.niucai.mapper.TaMatchMapper;
import com.yunzhi.niucai.mapper.TaMatchOddsMapper;
import com.yunzhi.niucai.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 比赛  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-02
 */
@Service
public class TaMatchServiceImpl extends ServiceImpl<TaMatchMapper, TaMatch> implements ITaMatchService {

    @Autowired
    ITaMatchWayService iTaMatchWayService;

    @Autowired
    ITaMatchOddsService iTaMatchOddsService;

    @Autowired
    ITaCustomerBettingService iTaCustomerBettingService;

    @Autowired
    ITaCustomerBettingItemService iTaCustomerBettingItemService;

    @Autowired
    ITaCustomerBettingSportService iTaCustomerBettingSportService;

    @Autowired
    ITaBettingPlanService iTaBettingPlanService;

    @Autowired
    ITdSportPassService iTdSportPassService;

    @Autowired
    TaMatchMapper taMatchMapper;

    @Autowired
    TaMatchOddsMapper taMatchOddsMapper;

    @Autowired
    TaCustomerBettingMapper taCustomerBettingMapper;

    @Override
    public List<TaMatch> getMixedMatch(String lottery) throws Exception {
        List<TaMatch> taMatches = taMatchMapper.getMixedMatch(lottery, SportStatusEnum.SALE.getCode());
        if (null == taMatches) {
            return null;
        }

        for (TaMatch match : taMatches) {
            List<Map<String, Object>> oddList = taMatchOddsMapper.getByMatch(match.getMatchId());
            if (null != oddList) {
                Map<String, List<Map<String, Object>>> odds = new HashMap<>();

                String key = "";
                List<Map<String, Object>> list = new ArrayList<>();
                for (Map<String, Object> item : oddList) {
                    String wayCode = item.get("wayCode").toString();
                    if (!key.equals(wayCode)) {
                        key = wayCode;
                        list = new ArrayList<>();
                        odds.put(key, list);
                    }
                    list.add(item);
                }

                Map<String, TaMatchWay> groupedByWay = iTaMatchWayService.getGroupedByWay(match.getMatchId(), lottery);

                match.setPlayOdds(odds);
                match.setMatchWay(groupedByWay);
            }
        }

        return taMatches;
    }

    @Override
    public List<TaMatch> getLastResult() throws Exception {
        return taMatchMapper.getLastResult();
    }

    @Override
    public IPage<TaMatch> getResult(IPage<TaMatch> page, String lotteryId) throws Exception {
        IPage<TaMatch> result = taMatchMapper.getResult(page, lotteryId);

        if (null != result.getRecords() && result.getRecords().size() > 0) {
            for (TaMatch taMatch : result.getRecords()) {

            }
        }

        return null;
    }

    /**
     * 体彩是否中奖
     * @throws Exception
     */
    public void checkAndSetSportPrize() throws Exception {
        List<TaMatch> matchList = taMatchMapper.getLastOpenAndUndoMatch(3);
        if (null == matchList || matchList.size() == 0) {
            return;
        }

        for (TaMatch taMatch : matchList) {
            try {
                taMatch.setOpeningStatus(SportOpeningStatusEnum.DOING.getCode());
                updateById(taMatch);
                validateMatch(taMatch);
            } catch (Exception e) {
                // TODO 比如错误日志记录
                taMatch.setOpeningStatus(SportOpeningStatusEnum.READY.getCode());
                updateById(taMatch);
            }
        }
    }

    /**
     * 校验比赛
     * @param taMatch
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    private void validateMatch (TaMatch taMatch) throws Exception {
        // 获取当前比赛的投注列表
        List<TaCustomerBettingItem> bettingItemList = iTaCustomerBettingItemService.getByMatch(taMatch);
        if (null == bettingItemList || bettingItemList.size() == 0) {
            return;
        }

        for (TaCustomerBettingItem item: bettingItemList) {
            if (!SportOpeningStatusEnum.READY.getCode().equals(item.getOpeningStatus())) {
                continue;
            }

            // 通过明细反向查询主记录
            // 但是主记录包含的明细不止当前这条
            TaCustomerBetting betting = iTaCustomerBettingService.getById(item.getBettingId());
            // 投注列表
            List<TaCustomerBettingSport> bettingSportList = iTaCustomerBettingSportService.getListByBetting(item.getBettingId(), item.getItemNo());
            // 判断是否中奖
            for (TaCustomerBettingSport bettingSport : bettingSportList) {
                boolean wining = isWining(taMatch, bettingSport);
                if (wining) {
                    bettingSport.setIsWinning(wining);
                    iTaCustomerBettingSportService.updateById(bettingSport);
                }
            }

            item.setOpeningStatus(SportOpeningStatusEnum.FINISHED.getCode());
            item.setSportList(bettingSportList);

            // 验证当前明细对应的主记录的所有明细是否都已经处理了
            checkAndSetBetting(betting, item);
        }
    }

    /**
     * 校验投注
     * @param betting
     * @param item
     * @throws Exception
     */
    private void checkAndSetBetting(TaCustomerBetting betting, TaCustomerBettingItem item) throws Exception {
        List<TaCustomerBettingItem> itemList = iTaCustomerBettingItemService.getByBetting(betting);
        if (null == itemList || itemList.size() == 0) {
            throw new Exception(String.format("投注[%s]没有找到任何投注明细", betting.getBettingId()));
        }

        boolean done = true;
        boolean existWinningRec = false;
        for (int i = 0; i < itemList.size(); i++) {
            TaCustomerBettingItem bettingItem = itemList.get(0);
            if (bettingItem.getItemNo().equals(item.getItemNo())) {
                itemList.set(i, item);

                if (item.getIsWinning()) {
                    existWinningRec = true;
                }

                continue;
            }

            if (bettingItem.getIsWinning()) {
                existWinningRec = true;
            }

            // 如果还有未处理完成的, 跳出循环
            if (!SportOpeningStatusEnum.FINISHED.getCode().equals(bettingItem.getOpeningStatus())) {
                done = false;
                break;
            }
        }

        // 如果还有场次没有结束, 那么不去算最终奖金
        if (!done) {
            return;
        }

        // 所有奖都开了
        betting.setIsOpen(true);
        betting.setOpeningDate(DateUtils.today());
        UpdateWrapper<TaCustomerBetting> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("is_open", betting.getIsOpen());
        updateWrapper.set("opening_date", betting.getOpeningDate());
        updateWrapper.eq("betting_id", betting.getBettingId());
        taCustomerBettingMapper.update(betting, updateWrapper);

        // 如果一场都没中, 直接 pass
        if (!existWinningRec) {
            return;
        }

        // 投注方案
        TaBettingPlan bettingPlan = iTaBettingPlanService.getById(betting.getPlanId());
        if (null == bettingPlan) {
            throw new Exception("校验投注方案失败");
        }
        betting.setBettingPlan(bettingPlan);

        // 结算最终奖金
        computePirzeAmount(betting, itemList);
    }

    /**
     * 结算最终奖金
     * 主要思路：
     * 1、先把所有场次组合一个数组, 其实就是 itemList
     * 2、依据所选的玩法, 把所有的玩法都组合出来, 比如4串5, 那么用场次把所有组合都穷举出来
     * 3、遍历所有组合, 计算每种组合的中奖金额
     * @param betting
     * @param itemList
     * @throws Exception
     */
    private void computePirzeAmount(TaCustomerBetting betting, List<TaCustomerBettingItem> itemList) throws Exception {
        // 参与算法的数据结构是一个两层List
        // List<                场次, 也就是最外层的 size == 场次数
        //      List<           投注名称, 内层的 size == 每场投注数
        //          String>>    当前投注的打单赔率, 不是投注赔率
        List<List<String>> noteList = new ArrayList<>();

        // 获取每个场次的玩法投注列表
        for (TaCustomerBettingItem item : itemList) {
            List<TaCustomerBettingSport> sportList = item.getSportList();

            if (null == sportList || sportList.size() == 0) {
                sportList = iTaCustomerBettingSportService.getListByBetting(betting.getBettingId(), item.getItemNo());
                if (null == sportList || sportList.size() == 0) {
                    continue;
                }
            }

            // 只有中奖的投注才会参与计算
            List<String> oddsOfNote = new ArrayList<>();
            for (TaCustomerBettingSport sportItem : sportList) {
                if (sportItem.getIsWinning()) {
                    oddsOfNote.add(sportItem.getOddsPrinted());
                } else {
                    // 未中奖的赔率是 0
                    oddsOfNote.add("0");
                }
            }

            noteList.add(oddsOfNote);
        }

        // 玩法字典
        if (null == betting.getBettingPlan().getPassCode()) {
            throw new Exception("校验过关模式失败");
        }
        TdSportPass tdSportPass = iTdSportPassService.getById(betting.getBettingPlan().getPassCode());
        if (null == tdSportPass) {
            throw new Exception("校验过关模式失败");
        }

        List<List<List<String>>> mockCombList = BizUtils.mockAllCombination(noteList, tdSportPass);

        // 计算方法:  https://www.lottery.gov.cn/bzzx/yxgz/20191119/1273604.html
        float totalCharge = 0.0f;
        for (List<List<String>> notes : mockCombList) {

            // 多种可能的结果会直接相加
            float perCombCharge = 0.0f;
            for (List<String> oddsList : notes) {

                // 比如购买的 3串4
                // 3串4的金额是 2串1 + 2串1 + 2串1 + 3串1
                float perNoteCharge = 0.0f;
                for (String oddsStr : oddsList) {

                    // 比如2串1
                    // 2串1的金额是  第一场比赛赔率*第二次比赛赔率
                    // 依次类推, 如果是 3串1 则是3场相乘
                    float odds = Float.parseFloat(oddsStr);

                    // 此处判断是否第一次循环, float 不能用 == 判断
                    if (perNoteCharge < 0.01f) {
                        perNoteCharge = odds;
                    } else {
                        perNoteCharge += odds;
                    }
                }

                perCombCharge += perNoteCharge;
            }

            // 金额 = 2块(200分) * 赔率 * 投注倍数
            totalCharge += 200 * perCombCharge * betting.getBettingTimes();
        }

        // 如果没有中奖
        if (totalCharge < 0.01f) {
            return;
        }

        betting.setIsWinning(true);
        betting.setWinAmount((int) totalCharge);
        UpdateWrapper<TaCustomerBetting> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("is_winning", betting.getIsWinning());
        updateWrapper.set("win_amount", betting.getWinAmount());
        updateWrapper.eq("betting_id", betting.getBettingId());
        taCustomerBettingMapper.update(betting, updateWrapper);

        // TODO 发送通知
    }

    private boolean isWining(TaMatch taMatch, TaCustomerBettingSport bettingSport) {
        TaMatchOdds taMatchOdds = iTaMatchOddsService.getByRule(taMatch.getMatchId(), bettingSport.getRuleCode());
        return null != taMatchOdds && taMatchOdds.getIsWinning();
    }
}
